/**
 * The ajax script.
 *
 * @since             0.1.0
 * @package           CB_Vegas
 * @subpackage        CB_Vegas/admin/menu/js
 * Author:            Demis Patti demis@demispatti.ch
 * Author URI:        http://demispatti.ch
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */

(function($){
	'use strict';

	$(document).ready(function(){

		/*--------------------------------------------------
		 * Cached objects
		 *------------------------------------------------*/
		var form = $('#cb-vegas-settings-form');
		var navigation = $('.cb-vegas-navigation-tabs-container');
		var slides = $('#cb-vegas-slides-container');
		var singleSlide = $('.cb-vegas-single-slide');
		var addMediaButton = $('.cb-vegas-add-media-button');
		var removeMediaButton = $('.cb-vegas-remove-media-button');
		var media = $('.cb-vegas-media');
		var notice = $('.wp-core-ui .notice.is-dismissible');
		var eventElements = [
			'#cb-vegas-add-slideshow-button',
			'.cb-vegas-remove-slideshow-button',
			'#cb-vegas-add-slide-button',
			'.cb-vegas-remove-slide-button',
			'.cb-vegas-add-media-button',
			'.cb-vegas-remove-media-button',
			'.cb-vegas-slide-handle',
			'#submit',
			'.cb-vegas-switch-container',
			'a'
		];
		var pointerEventsElements = $(eventElements);

		/*--------------------------------------------------
		 * Helper Functions
		 *------------------------------------------------*/
		function removeOptionsUpdatedNotice(){

			$('.wp-core-ui .notice.is-dismissible').animate({
				height: 0,
				duration: '600'
			}, 600).remove();
		}

		function disablePointerEvents(){

			pointerEventsElements.css('pointer-events', 'none');
		}

		function enablePointerEvents(){

			pointerEventsElements.css('pointer-events', 'auto');
		}

		function slideUpdateSlideIndices(){

			singleSlide.each(function(i){
				$(this).data('slide-index', i);
				$(this).attr('data-slide-index', i);
				$(this).attr('id', 'cb-vegas-single-slide_' + i);
			});
		}

		function updateNavTabLinks(){

			// Reassign the (tab-)indices to the links except for the first item.
			$('.nav-tab').each(function(i){

				if(i == 0){

					$(this).attr('href', '?page=cb_vegas_settings_page&tab=0');
				} else{

					$(this).attr('href', '?page=cb_vegas_settings_page&tab=' + i);
				}
			});
		}

		function updateNavTabIndices(){

			$('.cb-vegas-navigation-tab').each(function(i){

				$(this).data('slideshow-index', i);
				$(this).attr('data-slideshow-index', i);
				$(this).attr('id', 'cb-vegas-navigation-tab_' + i);
			});
		}

		function updateActiveTabIndex(){

			var form = $('#cb-vegas-settings-form');
			var activeTabIndex = $('.cb-vegas-navigation-tab-active-tab').data('slideshow-index');

			form.data('slideshow-index', activeTabIndex);
			form.attr('data-slideshow-index', activeTabIndex);
			$('#slideshow_index').prop('value', activeTabIndex);
		}

		function updateUrl(){

			var activeTabIndex = $('.cb-vegas-navigation-tab-active-tab').data('slideshow-index');
			var pathname = window.location.pathname;
			var settingsTab = '?page=cb_vegas_settings_page&tab=' + activeTabIndex;
			var newUrl = pathname + settingsTab;

			window.history.pushState(null, null, newUrl);
		}

		function updateSlideIndices(){

			$('.cb-vegas-single-slide').each(function(i){

				$(this).data('slide-index', i);
				$(this).attr('data-slide-index', i);
				$(this).attr('id', 'cb-vegas-single-slide_' + i);
			});
		}

		/*--------------------------------------------------
		 * Tooltips
		 *------------------------------------------------*/
		$(function(){
			$('#cb-vegas-add-slide-button').tooltip({
				content: {title: cbVegasAjax.addSlideButtonText},
				show: {effect: "blind", duration: 800},
				hide: {effect: "pulsate", duration: 1000},
				tooltipClass: "cb-vegas-tooltip",
				track: true
			});
			$('.cb-vegas-remove-slide-button').tooltip({
				content: {title: cbVegasAjax.removeSlideButtonText},
				show: {effect: "blind", duration: 800},
				hide: {effect: "pulsate", duration: 1000},
				tooltipClass: "cb-vegas-tooltip",
				track: true
			});
			$('#cb-vegas-add-tab-button').tooltip({
				content: {title: cbVegasAjax.addSlideshowButtonText},
				show: {effect: "blind", duration: 800},
				hide: {effect: "pulsate", duration: 1000},
				track: true
			});
			$('.cb-vegas-remove-tab-button').tooltip({
				content: {title: cbVegasAjax.removeSlideshowButtonText},
				show: {effect: "blind", duration: 800},
				hide: {effect: "pulsate", duration: 1000},
				track: true
			});
			$('.cb-vegas-duplicate-slide-button').tooltip({
				content: {title: cbVegasAjax.duplicateSlideButtonText},
				show: {effect: "blind", duration: 800},
				hide: {effect: "pulsate", duration: 1000},
				tooltipClass: "cb-vegas-tooltip",
				track: true
			});
			$('#cb-vegas-remove-slide-button').tooltip({
				content: {title: cbVegasAjax.removeSlideButtonText},
				show: {effect: "blind", duration: 800},
				hide: {effect: "pulsate", duration: 1000},
				tooltipClass: "cb-vegas-tooltip",
				track: true
			});
		});

		/*--------------------------------------------------
		 * Alertify Setp
		 *------------------------------------------------*/
		alertify.set({
			labels: {
				ok: cbVegasAjax.okText,
				cancel: cbVegasAjax.cancelText,
				buttonFocus: cbVegasAjax.cancelText
			},
			transition: 'pulse',
			delay: 6000
		});

		/*--------------------------------------------------
		 * Events
		 *------------------------------------------------*/
		$('.cb-vegas-remove-slide-button').live('click', function(event){

			event = event || window.event;
			event.preventDefault();

			disablePointerEvents();

			var data = {
				action: 'remove_slide',
				slideshow_index: $('#current_slideshow_index').val(),
				slide_index: $(this).parents().filter('.cb-vegas-single-slide').data('slide-index'),
				nonce: $(this).attr("data-nonce")
			};

			alertify.confirm(cbVegasAjax.removeSlideConfirmation, function(event){

				if(event){

					$.post(ajaxurl, data, function(response){

						//alert(response);

						if(response.data.success == true){

							// Removes the element containing the slide.
							singleSlide.filter('[data-slide-index=' + data.slide_index + ']').remove();

							// Reassigns the index in ascending order.
							slideUpdateSlideIndices();

							alertify.success(response.data.message);
							enablePointerEvents();
							removeOptionsUpdatedNotice();
							return true;
						} else{

							alertify.error(response.data.message);
							enablePointerEvents();
							return false;
						}
					});
				} else{

					alertify.error(cbVegasAjax.actionCancelledMessage);
					enablePointerEvents();
					return false;
				}
			});
		});

		$('#cb-vegas-add-slide-button').on('click', function(event){

			event = event || window.event;
			event.preventDefault();

			disablePointerEvents();

			// Determine the highest index number.
			var indices = $('.cb-vegas-single-slide').map(function(i, element){
				return $(element).data('slide-index');
			});
			var highestIndex = Math.max.apply(Math, indices);

			var slide_index = null;

			if($.isNumeric(highestIndex)){
				slide_index = highestIndex + 1;
			} else{
				slide_index = 0;
			}

			var data = {
				action: 'add_slide',
				slideshow_index: $('#current_slideshow_index').val(),
				slide_index: slide_index,
				nonce: $(this).attr('data-nonce')
			};

			alertify.confirm(cbVegasAjax.addSlideConfirmation, function(event){

				if(event){

					$.post(ajaxurl, data, function(response){

						//alert(response);

						if(response.data.success == true){

							var slide = response.data.html;
							// Appends the slide.
							slides.append(slide);

							addMediaButton.filter('[data-slide-index="' + data.slide_index + '"]').css('visibility', 'visible').hide();
							removeMediaButton.filter('[data-slide-index="' + data.slide_index + '"]').css('visibility', 'visible').show();

							$('.cb-vegas-add-media-button').filter('[data-slide-index="' + data.slide_index + '"]').css({
								//visibility: ' hidden',
								display: 'none'
							});

							$('.cb-vegas-remove-media-button').filter('[data-slide-index="' + data.slide_index + '"]').css({
								//visibility: 'visible',
								display: 'inline-block'

							});

							// Switchifies the newly created checkboxes.
							$("<div class='cb-vegas-switch-container'></div>").insertAfter('.cb-vegas-switch');

							// Updates the newly created select boxes.
							$('.cb-vegas-fancy-select').fancySelect();

							// Updates the newly created color picker.
							$('.cb-vegas-color-picker').wpColorPicker();

							alertify.success(response.data.message);
							enablePointerEvents();
							removeOptionsUpdatedNotice();
							return true;
						} else{

							alertify.error(response.data.message);
							enablePointerEvents();
							return false;
						}
					});
				} else{

					alertify.error(cbVegasAjax.actionCancelledMessage);
					enablePointerEvents();
					return false;
				}
			});
		});

		$('#cb-vegas-remove-slideshow-button').on('click', function(event){
			
			event = event || window.event;
			event.preventDefault();
			
			disablePointerEvents();

			var data = {
				action: 'remove_slideshow',
				deprecated_slideshow_index: $('#current_slideshow_index').val(),
				nonce_key: $(this).data('nonce-key'),
				nonce: $(this).attr('data-nonce')
			};
			
			alertify.confirm(cbVegasAjax.removeSlideshowConfirmation, function(event){

				if(event){
					
					$.post(ajaxurl, data, function(response){
						
						//alert(response);
						
						if(response.data.success === true){

							enablePointerEvents();
							removeOptionsUpdatedNotice();
							
							alertify.success(response.data.message);
							
							var tab = (data.deprecated_slideshow_index > 0) ? data.deprecated_slideshow_index - 1 : 0;
							window.location.href = '/wp-admin/options-general.php?page=cb_vegas_settings_page&tab=' + tab;
							return true;
						} else{
							
							enablePointerEvents();

							alertify.error(response.data.message);
							return false;
						}
					});
				} else{
					
					enablePointerEvents();
					alertify.error(cbVegasAjax.actionCancelledMessage);
					return false;
				}
			});
		});
		
		$('#cb-vegas-add-tab-button').on('click', function(event){
			
			event = event || window.event;
			event.preventDefault();
			
			disablePointerEvents();

			var data = {
				action: 'add_slideshow',
				nonce: $(this).attr('data-nonce')
			};
			
			alertify.confirm(cbVegasAjax.addSlideshowConfirmation, function(event){
				if(event){
					$.post(ajaxurl, data, function(response){

						//alert(response);
						
						if(response.data.success == true){
							
							var tab = response.data.html;
							
							// Appends the slide.
							$('.cb-vegas-navigation-tabs-container').append(tab);
							
							// Switchifies the newly created checkboxes.
							$("<div class='cb-vegas-switch-container'></div>").insertAfter('.cb-vegas-switch');
							
							// Updates "Fancy Select".
							$('.cb-vegas-fancy-select').trigger('update.fs');
							
							updateNavTabLinks();
							updateNavTabIndices();
							
							enablePointerEvents();
							removeOptionsUpdatedNotice();
							alertify.success(response.data.message);
							return true;
						} else{
							
							enablePointerEvents();
							alertify.error(response.data.message);
							return false;
						}
					});
				} else{
					
					enablePointerEvents();
					alertify.error(cbVegasAjax.actionCancelledMessage);
					return false;
				}
			});
		});

		/*$('.cb-vegas-navigation-tabs-container').sortable({
			containment: 'parent',
			cursor: 'move',
			delay: 150,
			forceHelperSize: true,
			forcePlaceholderSize: true,
			axis: 'x',
			
			update: function(){
				
				disablePointerEvents();

				var data = {
					action: 'sort_slideshows',
					indices: $(this).sortable('toArray'),
					nonce: $(this).attr("data-nonce"),
					slideshow_index: $('#current_slideshow_index').val()
				};
				
				$.post(ajaxurl, data, function(response){
					
					if(response.data.success == true){
						
						// Reassign the indices of the nav tabs in ascending order.
						updateNavTabIndices();
						
						// Assign the new tab-index to the form.
						updateActiveTabIndex();
						
						// Reassign the (tab-)indices to the links except for the first item.
						updateNavTabLinks();
						
						updateUrl();
						
						//alertify.success(response.data.message);
						enablePointerEvents();
						removeOptionsUpdatedNotice();
						return true;
					} else{
						
						alertify.error(response.data.success);
						enablePointerEvents();
						return false;
					}
				});
			}
		});*/
		
		$('#cb-vegas-slides-container').sortable({
			containment: 'parent',
			cursor: 'move',
			delay: 150,
			forceHelperSize: true,
			forcePlaceholderSize: true,
			
			update: function(){
				
				disablePointerEvents();

				var data = {
					action: 'sort_slides',
					slideshow_index: $('#current_slideshow_index').val(),
					indices: $(this).sortable('toArray'),
					nonce: $(this).attr("data-nonce")
				};
				
				$.post(ajaxurl, data, function(response){
					
					if(response.data.success == true){
						
						updateSlideIndices();
						
						alertify.success(response.data.message);
						enablePointerEvents();
						removeOptionsUpdatedNotice();
						return true;
					} else{
						
						alertify.error(response.data.success);
						enablePointerEvents();
						return false;
					}
				});
			}
		});

		// @todo: grab vals from the actual input fields with js and validate / escape values with a fallback to the default values.
		$('.cb-vegas-duplicate-slide-button').live('click', function(event){
			
			event = event || window.event;
			event.preventDefault();
			
			disablePointerEvents();

			var slide_index = $(this).data('slide-index');
			var slideshow_index = $('#current_slideshow_index').val();
			
			var data = {
				action: 'duplicate_slide',
				slideshow_index: slideshow_index,
				slide_index: slide_index,
				nonce: $(this).attr('data-nonce')
			};
			
			alertify.confirm(cbVegasAjax.duplicateSlideConfirmation, function(event){
				
				if(event){
					
					$.post(ajaxurl, data, function(response){
						
						//alert(response);
						
						if(response.data.success == true){
							
							var slide = response.data.html;
							var newSlideIndex = data.slide_index + 1;
							
							// Appends the slide.
							slides.append(slide);
							
							// Sets the initial view.
							addMediaButton.filter('[data-slide-index="' + newSlideIndex + '"]').css('visibility', 'visible').hide();
							removeMediaButton.filter('[data-slide-index="' + newSlideIndex + '"]').css('visibility', 'visible').show();
							
							$('.cb-vegas-add-media-button').filter('[data-slide-index="' + newSlideIndex + '"]').css({
								display: 'none'
							});
							
							$('.cb-vegas-remove-media-button').filter('[data-slide-index="' + newSlideIndex + '"]').css({
								display: 'inline-block'
								
							});
							
							// Switchifies the newly created checkboxes.
							$("<div class='cb-vegas-switch-container'></div>").insertAfter('.cb-vegas-switch');
							
							// Updates the newly created select boxes.
							$('.cb-vegas-fancy-select').fancySelect();
							
							// Updates the newly created color picker.
							$('.cb-vegas-color-picker').wpColorPicker();
							
							alertify.success(response.data.message);
							
							slideUpdateSlideIndices();
							enablePointerEvents();
							removeOptionsUpdatedNotice();
							return true;
						} else{
							alertify.error(response.data.message);
							enablePointerEvents();
							return false;
						}
					});
				} else{
					alertify.error(cbVegasAjax.actionCancelledMessage);
					enablePointerEvents();
					return false;
				}
			});
		});

		// @notice: using the values from the current options transient is okay for now.
		$('#cb-vegas-duplicate-slideshow-button').on('click', function(event){

			event = event || window.event;
			event.preventDefault();

			disablePointerEvents();

			var slideIndex = $(this).data('slide-index');
			var slideshowIndex = $('#current_slideshow_index').val();

			var data = {
				action: 'duplicate_slideshow',
				slideshow_index: slideshowIndex,
				slide_index: slideIndex,
				nonce: $(this).attr('data-nonce')
			};

			alertify.confirm(cbVegasAjax.dupliacteSlideshowConfirmation, function(event){

				if(event){

					$.post(ajaxurl, data, function(response){

						//alert(response);

						if(response.data.success == true){

							var navigationTab = response.data.html;

							// Appends the slide.
							$('.cb-vegas-navigation-tabs-container').append(navigationTab);

							// Updates "Fancy Select".
							$('.cb-vegas-fancy-select').trigger('update.fs');

							alertify.success(response.data.message);
							enablePointerEvents();
							removeOptionsUpdatedNotice();
							return true;
						} else{

							alertify.error(response.data.message);
							enablePointerEvents();
							return false;
						}
					});
				} else{

					alertify.error(cbVegasAjax.actionCancelledMessage);
					enablePointerEvents();
					return false;
				}
			});
		});
		/*--------------------------------------------------
		 * Init Slideshow Index And Tab Number
		 *------------------------------------------------*/
		updateActiveTabIndex();
		updateUrl();
	});
	
})(jQuery);
