/**
 * The main script.
 *
 * @since             0.1.0
 * @package           CB_Vegas
 * @subpackage        CB_Vegas/admin/menu/js
 * Author:            Demis Patti demis@demispatti.ch
 * Author URI:        http://demispatti.ch
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */

(function($){
	'use strict';

	$(document).ready(function(){

		$('#cb-vegas-settings-form input, div.fancy-select ul.options li.selected').on('change', function(){
			$('#cb-vegas-duplicate-slideshow-button').css({
				//boxShadow: 'none',
				//textShadow: 'none',
				//backgroundColor: '#F7F7F7',
				pointerEvents: 'none'
			})/*.attr('title', 'Save your settings first!')*/;
			$("#cb-vegas-duplicate-slideshow-button").tooltip({
				content: {title: cbVegas.duplicateSlideshowSaveSettingsFirstButtonText},
				show: {effect: "blind", duration: 800},
				hide: {effect: "pulsate", duration: 1000},
				tooltipClass: "cb-vegas-tooltip",
				track: true
			});
		});

		// Wraps the "settings updated notice" and assigns its height to the wrap to ensure that the notice is dismissible more eye-candy-like.
		if($('#setting-error-settings_updated').length){
			$(this).css('display', 'none');
			$('#setting-error-settings_updated').wrapAll('<div class="setting-error-settings_updated_placeholder"></div>');
			$('.setting-error-settings_updated_placeholder').height($('#setting-error-settings_updated').outerHeight());
			//$('.setting-error-update-settings').append('<p><strong>Don\'t forget to save your settings.</strong><p><button type="button" class="notice-dismiss"><span class="screen-reader-text">Dismiss this notice.</span></button>');
			$(this).css('display', 'initial');
		}

		/*--------------------------------------------------
		 * Color Picker
		 *------------------------------------------------*/
		$('.cb-vegas-color-picker').wpColorPicker();

		/*--------------------------------------------------
		 * Fancy Select
		 *------------------------------------------------*/
		$('.cb-vegas-fancy-select').fancySelect();

		/*--------------------------------------------------
		 * Localisation ( Switches )
		 *------------------------------------------------*/
		if(cbVegas.locale != 'default'){

			$('<style>:root input[type="checkbox"].cb-vegas-switch + div:before{content:"' + cbVegas.onText + '";}</style>').appendTo('head');
			$('<style>:root input[type="checkbox"].cb-vegas-switch + div:after{content:"' + cbVegas.offText + '";}</style>').appendTo('head');
		}

		/*--------------------------------------------------
		 * Settings Container Toggle
		 *------------------------------------------------*/
		var toggle = $('.cb-vegas-settings-toggle');
		var settingsContainer = $('.cb-vegas-settings-slider');

		settingsContainer.css({display: 'none'});
		toggle.click(function(event){
			event = event || window.event;
			event.preventDefault();

			settingsContainer.slideToggle(600);
		});

		/*--------------------------------------------------
		 * Add remove media functions
		 *------------------------------------------------*/
		function removeOptionsUpdatedNotice(){
			var notice = $('.wp-core-ui .notice.is-dismissible');

			notice.animate({
				height: 0,
				duration: '600'
			}, 600).remove();
		}

		/*--------------------------------------------------
		 * jQuery UI ToolTip ( Buttons )
		 *------------------------------------------------*/
		$(function(){
			$(".cb-vegas-add-media-button").tooltip({
				content: {title: cbVegas.addMediaButtonText},
				show: {effect: "blind", duration: 800},
				hide: {effect: "pulsate", duration: 1000},
				tooltipClass: "cb-vegas-tooltip",
				track: true
			});
			$(".cb-vegas-remove-media-button").tooltip({
				content: {title: cbVegas.removeMediaButtonText},
				show: {effect: "blind", duration: 800},
				hide: {effect: "pulsate", duration: 1000},
				tooltipClass: "cb-vegas-tooltip",
				track: true
			});
		});

		/*--------------------------------------------------
		 * Media Uploader ( Image )
		 *------------------------------------------------*/
		var image = $('.cb-vegas-media');

		// Sets the initial view
		if(image.attr('src')){

			$('.cb-vegas-add-media-button').css('visibility', 'visible').hide();
			$('.cb-vegas-media').css('visibility', 'visible').show();
			$('.cb-vegas-remove-media-button').css('visibility', 'visible').show();
		} else{

			$('.cb-vegas-media').hide();
			$('.cb-vegas-remove-media-button').css('visibility', 'visible').hide();
			$('.cb-vegas-add-media-button').css('visibility', 'visible').show();
		}

		// Removes media file.
		/*removeMediaButton*/
		$('.cb-vegas-remove-media-button').live('click', function(event){
			event = event || window.event;
			event.preventDefault();

			var slide_index = $(this).data('slide-index');
			$('.cb-vegas-media').filter('[data-slide-index="' + slide_index + '"]').attr('src', '').hide();
			$('.cb-vegas-media-hidden').filter('[data-slide-index="' + slide_index + '"]').val('');
			$('.cb-vegas-remove-media-button').filter('[data-slide-index="' + slide_index + '"]').hide();
			$('.cb-vegas-add-media-button').filter('[data-slide-index="' + slide_index + '"]').show();
			// reset.
			slide_index = null;
			removeOptionsUpdatedNotice();
		});

		// Adds media file.
		var cb_vegas_frame;

		/*addMediaButton*/
		$('.cb-vegas-add-media-button').live('click', function(event){
			event = event || window.event;
			event.preventDefault();

			var slide_index = $(this).data('slide-index');

			if(cb_vegas_frame){
				cb_vegas_frame.open(slide_index);
				return;
			}
			cb_vegas_frame = wp.media.frames.cb_vegas_frame = wp.media({

				className: "media-frame cb-vegas-frame",
				frame: "select",
				multiple: false,
				title: cbVegas.frameTitleText,
				library: {type: ['image', 'video']},
				button: {text: cbVegas.frameButtonText},
				index: slide_index
			});

			// Processes the selected image
			cb_vegas_frame.on("select", function(){
				// Gets the selected item.
				var mediaAttachment = cb_vegas_frame.state().get('selection').first().toJSON();

				$('.cb-vegas-add-media-button').filter('[data-slide-index="' + slide_index + '"]').hide();
				$('.cb-vegas-media').filter('[data-slide-index="' + slide_index + '"]').attr('src', mediaAttachment.url).show();
				$('.cb-vegas-media-hidden').filter('[data-slide-index="' + slide_index + '"]').val(mediaAttachment.url);
				$('.cb-vegas-remove-media-button').filter('[data-slide-index="' + slide_index + '"]').css('visibility', 'visible').show();
				// reset
				cb_vegas_frame = null;
				removeOptionsUpdatedNotice();
			});
			cb_vegas_frame.open();
		});
	});

})(jQuery);
