/**
 * Vegas Slideshow Configuration.
 *
 * @since             0.1.0
 * @package           CB_Vegas
 * @subpackage        CB_Vegas/public/js
 * Author:            demispatti <demis@demispatti.ch>
 * Author URI:        http://demispatti.ch
 * License:           GPL-2.0+
 * License URI:       http://www.gnu.org/licenses/gpl-2.0.txt
 */

(function($){
	'use strict';

	var body = $('body');
	var customSlideSettings = [];
	var customSlideshowSettings = [];
	var overlay_path = null;
	var vegasOverlay = null;
	var slidesList = [];

	var shuffle = Vegas.meta.slideshow_is_shuffle;
	var overlay = Vegas.meta.slideshow_is_overlay != 'none' ? Vegas.meta.slideshow_is_overlay : false;
	var autoplay = Vegas.meta.slideshow_is_autoplay;
	var timer = Vegas.meta.slideshow_is_timer;



	function setSlideSettings(){

		$.each(Vegas.slides, function(i){
			customSlideSettings.push({
				src: Vegas.settings[i].src,
				slideDelay: Vegas.settings[i].slideDelay,
				preload: Vegas.settings[i].preload,
				preloadImage: Vegas.settings[i].preloadImage,
				preloadVideo: false,
				cover: Vegas.settings[i].cover,
				color: Vegas.settings[i].color,
				align: Vegas.settings[i].align,
				valign: Vegas.settings[i].valign,
				transition: Vegas.settings[i].transition,
				transitionDuration: Vegas.settings[i].transitionDuration,
				animation: Vegas.settings[i].animation,
				animationDuration: Vegas.settings[i].animationDuration
			});
		});

		return customSlideSettings;
	}

	function setSlideshowSettings(){

		customSlideshowSettings = {
			autoplay: Vegas.meta.slideshow_is_autoplay,
			shuffle: Vegas.meta.slideshow_is_shuffle,
			timer: Vegas.meta.slideshow_is_timer,
			overlay: Vegas.overlay_path + Vegas.meta.slideshow_is_overlay
		};

		return customSlideshowSettings;
	}

	function setSiteUrl(){

		overlay_path = Vegas.overlay_path;
	}

	function setOverlayContainer(){

		vegasOverlay = $('.vegas-overlay');
	}

	function loadSlides(){

		slidesList = setSlideSettings();
	}

	function run(){

		setSlideSettings();
		setSlideshowSettings();
		setSiteUrl();
		setOverlayContainer();
		loadSlides();

		body.vegas({

			overlay: overlay_path + overlay,
			autoplay: autoplay,
			timer: timer,
			slides: shuffle ? slidesList.sort(function(){return 0.5 - Math.random()}) : slidesList,

			walk: function(index, slideSettings){

				slideSettings.src = customSlideSettings[index].src;
				slideSettings.delay = parseInt(customSlideSettings[index].slideDelay);
				//slideSettings.shuffle = shuffle;
				slideSettings.preload = customSlideSettings[index].preload;
				slideSettings.preloadImage = customSlideSettings[index].preloadImage;
				slideSettings.preloadVideo = customSlideSettings[index].preloadVideo;
				slideSettings.cover = customSlideSettings[index].cover;
				slideSettings.color = customSlideSettings[index].color;
				slideSettings.align = customSlideSettings[index].align;
				slideSettings.valign = customSlideSettings[index].valign;
				slideSettings.transition = customSlideSettings[index].transition;
				slideSettings.transitionDuration = parseInt(customSlideSettings[index].transitionDuration);
				slideSettings.animation = customSlideSettings[index].animation;
				slideSettings.animationDuration = parseInt(customSlideSettings[index].animationDuration);
			}
		});
	}

	$(document).ready(function(){

		run();

	});

})(jQuery);
